'use strict';

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define([], function () {
	/**
  * A Column in a table.
  * It contains the (header) label of the column and its unique name.
  * There are two specializations of a Column:
  *  - MeasureColumn for a column containing numerical values.
  *  - CategoricalColumn for a column containing discrete (string) values, encoded by a Tuple.
  */
	var Column = function () {
		function Column(_type, _label, _id, _kind) {
			_classCallCheck(this, Column);

			Object.defineProperties(this, {
				type: {
					enumerable: true,
					value: _type
				},
				label: {
					enumerable: true,
					value: _label
				},
				id: {
					enumerable: true,
					value: _id
				},
				kind: {
					enumerable: true,
					value: _kind || null
				}
			});
		}

		Column.prototype.toString = function toString() {
			return this.label;
		};

		return Column;
	}();

	/**
  * Select columns of kind `'input'`.
  * @param {Column} _column The column to inspect.
  * @returns {boolean} Whether the column has the kind `'input'`.
  */


	function isInputColumn(_column) {
		return _column.kind === 'input';
	}

	/**
  * Select columns of kind `'target'`.
  * @param {Column} _column The column to inspect.
  * @returns {boolean} Whether the column has the kind `'target'`.
  */
	function isTargetColumn(_column) {
		return _column.kind === 'target';
	}

	/**
  * Select columns of kind `'context'`.
  * @param {Column} _column The column to inspect.
  * @returns {boolean} Whether the column has the kind `'context'`.
  */
	function isContextColumn(_column) {
		return _column.kind === 'context';
	}

	return {
		isInputColumn: isInputColumn,
		isTargetColumn: isTargetColumn,
		isContextColumn: isContextColumn,

		isColumn: function isColumn(_column) {
			return _column instanceof Column;
		},
		MeasureColumn: function (_Column) {
			_inherits(MeasureColumn, _Column);

			function MeasureColumn(_label, _id, _kind, _aggregation, _formatter) {
				_classCallCheck(this, MeasureColumn);

				var _this = _possibleConstructorReturn(this, _Column.call(this, 'measure', _label, _id, _kind));

				Object.defineProperties(_this, {
					aggregation: {
						enumerable: true,
						value: _aggregation
					},
					formatter: {
						enumerable: true,
						value: _formatter
					}
				});
				return _this;
			}

			return MeasureColumn;
		}(Column),

		CategoricalColumn: function (_Column2) {
			_inherits(CategoricalColumn, _Column2);

			function CategoricalColumn(_label, _id, _kind, _tuples) {
				_classCallCheck(this, CategoricalColumn);

				var _this2 = _possibleConstructorReturn(this, _Column2.call(this, 'categorical', _label, _id, _kind));

				Object.defineProperty(_this2, 'tuples', {
					enumerable: true,
					value: _tuples
				});
				return _this2;
			}

			return CategoricalColumn;
		}(Column)

	};
});
//# sourceMappingURL=Column.js.map