'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['./Column', './Decorated'], function (Column, Decorated) {
	var msg_scenario = ''; //'Scenario';
	var msg_base_scenario = 'Baseline';

	var nextScenarioId = 0;

	/** Special scenario instance used to represent the base scenario. It has no overridden data */
	var BASE_SCENARIO = new Decorated.Scenario(msg_base_scenario, 'scenario.base', null, null);

	/**
  * DeltaTable creates a view of the Base (original data) combined with a new scenario.
  * To preserve memory, it doesn't actually create a single merged view, instead it tracks the overriding inputs and
  * results.
  */

	var DeltaTable = function () {
		function DeltaTable(_sourceColumns, _rowData) {
			_classCallCheck(this, DeltaTable);

			var columns = new Array(_sourceColumns.length + 1);
			/** scenarios is also used as tuples array for our virtual categorical scenarios array */
			var scenarios = [];
			scenarios.push(BASE_SCENARIO);
			// Insert a scenario categorical column as first column
			columns[0] = new Column.CategoricalColumn(msg_scenario, 'scenario', 'scenario', scenarios);
			// Copy existing data items to index 1 and up.
			for (var i = 0; i < _sourceColumns.length; i++) {
				columns[i + 1] = _sourceColumns[i];
			}

			Object.defineProperties(this, {
				columns: {
					enumerable: true,
					value: Object.freeze(columns)
				},
				_rowData: {
					value: _rowData
				},
				_scenarios: {
					value: scenarios
				}
			});
		}

		DeltaTable.prototype.getScenarioAt = function getScenarioAt(_index) {
			return this._scenarios[_index < 0 ? this._scenarios.length + _index : _index] || null;
		};

		DeltaTable.prototype.addScenario = function addScenario(_caption, _inputs, _results) {
			if (!_inputs || !_results) {
				throw new TypeError('A scenario must have inputs and results');
			}
			var scenario = new Decorated.Scenario(_caption, 'scenario.scen' + ++nextScenarioId, _inputs, _results);
			this._scenarios.push(scenario);
			return scenario;
		};

		DeltaTable.prototype.removeScenario = function removeScenario(_scenario) {
			var scenarioIndex = this._scenarios.indexOf(_scenario);
			// Do not allow removing the base scenario, e.g. exclude index 0.
			if (scenarioIndex > 0) {
				this.removeScenarioAt(scenarioIndex);
				return true;
			}
			return false;
		};

		DeltaTable.prototype.removeScenarioAt = function removeScenarioAt(_index) {
			var scenarioIndex = _index < 0 ? this._scenarios.length + _index : _index;
			if (scenarioIndex > 0 && scenarioIndex < this._scenarios.length) {
				return this._scenarios.splice(scenarioIndex, 1)[0];
			}
			return null;
		};

		DeltaTable.prototype.clearScenarios = function clearScenarios() {
			// The "base" Scenario and its Tuple must be preserved
			this._scenarios.length = 1;
		};

		/**
   * Iterate all rows with a call to callback per row.
   * @param {function} _callback Callback called with: `_row, _columns, _rowIndex`.
   * Warning: the row Array is re-used and updated to preserve memory. Values must be copied to preserve them.
   */


		DeltaTable.prototype.iterate = function iterate(_callback) {
			var sourceColumnCount = this.columns.length - 1;
			var row = new Array(sourceColumnCount + 1);
			var inputOffset = this.columns.findIndex(Column.isInputColumn);
			var targetIndex = this.columns.findIndex(Column.isTargetColumn);
			/** Warning: includes the virtual "base" scenario */
			var scenarioCount = this._scenarios.length;
			var rowIndex = 0;
			for (var baseRowIndex = 0, rowOffset = 0; rowOffset < this._rowData.length; baseRowIndex++, rowOffset += sourceColumnCount) {
				// Select the base scenario
				row[0] = 0;
				// copy the source row
				for (var i = 0; i < sourceColumnCount; i++) {
					row[i + 1] = this._rowData[rowOffset + i];
				}
				// Invoke the callback for the "base" row
				_callback(row, this.columns, rowIndex++, baseRowIndex);

				// start at index 1, scenario 0 is the virtual base scenario.
				for (var scenarioIndex = 1; scenarioIndex < scenarioCount; scenarioIndex++) {
					// The scenarios Array does not contain a "base" scenario, adjust the indexs
					var scenario = this._scenarios[scenarioIndex];
					if (baseRowIndex >= scenario.inputs.length) {
						break;
					}
					// Select the scenario tuple
					row[0] = scenarioIndex;

					// Apply the Scenario's input values
					var inputs = scenario.inputs[baseRowIndex];
					for (var _i = 0; _i < scenario.inputs[0].length; _i++) {
						row[inputOffset + _i] = inputs[_i];
					}
					// Apply the Scenario's target value
					row[targetIndex] = scenario.results[baseRowIndex];
					// Invoke the callback for the Secnario row
					_callback(row, this.columns, rowIndex++, baseRowIndex);
				}
			}
		};

		_createClass(DeltaTable, [{
			key: 'scenarios',
			get: function get() {
				// Create a frozen copy of the internal Array, so it is clear this array should not be modified externally.
				return Object.freeze(this._scenarios.slice(0));
			}
		}, {
			key: 'columnCount',
			get: function get() {
				return this.columns.length;
			}

			/**
    * Get the row count of the source data (e.g. the base scenario).
    */

		}, {
			key: 'baseRowCount',
			get: function get() {
				return this._rowData.length / (this.columns.length - 1);
			}

			/**
    * Get the full row count including all scenarios.
    */

		}, {
			key: 'rowCount',
			get: function get() {
				return this.baseRowCount * this.scenarioCount;
			}

			/**
    * The amount of scenarios in the DeltaTable.
    * Always includes the base scenario, e.g. after clear this returns `1`.
    */

		}, {
			key: 'scenarioCount',
			get: function get() {
				return this._scenarios.length;
			}
		}]);

		return DeltaTable;
	}();

	// return the object that defines this module


	return {
		/**
   * This method will take a source table, input deltas and a results Array, and create a virtual view that
   * combines the sources and the predicted rows.
   * @param {SourceTable} _sourceTable The SourceTable containing the base data.
   */
		createDeltaTable: function createDeltaTable(_sourceTable) {
			return new DeltaTable(_sourceTable.columns, _sourceTable.getRowData());
		}
	};
});
//# sourceMappingURL=DeltaTable.js.map