'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['@waca/dashboard-common/js/core/APIFactory', './WhatIfAPI', '@waca/dashboard-livewidget/js/features/content/dataQueryExecution/api/QueryModifierAPI'], function (APIFactory, WhatIfAPI, QueryModifierAPI) {
	var WhatIfFeatures = function () {
		function WhatIfFeatures(_ref) {
			var spec = _ref.spec,
			    content = _ref.content,
			    features = _ref.features;

			_classCallCheck(this, WhatIfFeatures);

			this._api = APIFactory.createAPI(this, [WhatIfAPI, QueryModifierAPI]);
			this._content = content;
			this._features = features.DataQueryExecution.registerQueryModifier(this.getAPI());
			this._scenario = spec && spec.scenario ? spec.scenario : null;
		}

		WhatIfFeatures.prototype.getAPI = function getAPI() {
			return this._api;
		};

		WhatIfFeatures.prototype.getType = function getType() {
			return 'main';
		};

		WhatIfFeatures.prototype.setScenario = function setScenario(scenario) {
			this._scenario = { scenario: scenario };
		};

		WhatIfFeatures.prototype.toJSON = function toJSON() {
			return this._scenario;
		};

		WhatIfFeatures.prototype.modifyQuerySpecList = function modifyQuerySpecList(querySpecList) {
			// Always Remove Aggregation 
			if (this._isTypeWhatIf()) {
				//if (this._isTypeWhatIf() && this._shouldRemoveAggregation()) {
				this._removeAggregation(querySpecList);
				this._addDetails(querySpecList);
			}
			return querySpecList;
		};

		WhatIfFeatures.prototype._isTypeWhatIf = function _isTypeWhatIf() {
			return this._content.getFeature('Visualization').getType() === 'WhatIf';
		};

		WhatIfFeatures.prototype._shouldRemoveAggregation = function _shouldRemoveAggregation() {
			return !this._isContextSlotMapped() || this._isContextSlotMapped() && this.__hasStringDataItemInInputSlot();
		};

		WhatIfFeatures.prototype._removeAggregation = function _removeAggregation(querySpecList) {
			querySpecList.forEach(function (querySpec) {
				querySpec.spec.dataItems.forEach(function (dataItem) {
					if (dataItem.aggregate) {
						dataItem.aggregate = 'none';
					}
				});
			});
		};

		WhatIfFeatures.prototype._addDetails = function _addDetails(querySpecList) {
			querySpecList.forEach(function (querySpec) {
				querySpec.spec.type = 'detail';
			});
		};

		WhatIfFeatures.prototype._isContextSlotMapped = function _isContextSlotMapped() {
			var result = false;
			var mappedSlots = this._content.getFeature('Visualization').getSlots().getMappedSlotList();
			mappedSlots.forEach(function (slot) {
				if (slot.getId() === 'context') {
					result = true;
				}
			});
			return result;
		};

		WhatIfFeatures.prototype.__hasStringDataItemInInputSlot = function __hasStringDataItemInInputSlot() {
			var result = false;
			var mappedSlots = this._content.getFeature('Visualization').getSlots().getMappedSlotList();
			var index = mappedSlots.findIndex(function (slot) {
				return slot.getId() === 'input';
			});
			var inputSlot = mappedSlots[index];
			inputSlot.getDataItemList().forEach(function (dataItem) {
				if (dataItem.getDataType() === 'string') {
					result = true;
				}
			});
			return result;
		};

		return WhatIfFeatures;
	}();

	return WhatIfFeatures;
});
//# sourceMappingURL=WhatIfFeatures.js.map